@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 from six.moves import range  # noqa
 
 from os_win import constants
@@ -60,6 +61,12 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
     _FAKE_DYNAMIC_MEMORY_RATIO = 1.0
     _FAKE_MONITOR_COUNT = 1
 
+    _FAKE_MEMORY_INFO = {'DynamicMemoryEnabled': True,
+                         'Reservation': 1024,
+                         'Limit': 4096,
+                         'Weight': 5000,
+                         'MaxMemoryBlocksPerNumaNode': 2048}
+
     _FAKE_SUMMARY_INFO = {'NumberOfProcessors': 4,
                           'EnabledState': 2,
                           'MemoryUsage': 2,
@@ -144,6 +151,21 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
     def test_set_vm_memory_dynamic(self):
         self._test_set_vm_memory_dynamic(dynamic_memory_ratio=2.0)
 
+    @mock.patch.object(_wqlutils, 'get_element_associated_class')
+    def test_get_vm_memory_info(self, mock_get_element_associated_class):
+        vmsetting = self._lookup_vm()
+
+        mock_s = mock.MagicMock(**self._FAKE_MEMORY_INFO)
+        mock_get_element_associated_class.return_value = [mock_s]
+
+        memory = self._vmutils.get_vm_memory_info(self._FAKE_VM_NAME)
+        self.assertEqual(self._FAKE_MEMORY_INFO, memory)
+
+        mock_get_element_associated_class.assert_called_once_with(
+            self._vmutils._compat_conn,
+            self._vmutils._MEMORY_SETTING_DATA_CLASS,
+            element_instance_id=vmsetting.InstanceID)
+
     @mock.patch.object(_wqlutils, 'get_element_associated_class')
     def _test_set_vm_memory_dynamic(self, mock_get_element_associated_class,
                                     dynamic_memory_ratio,
@@ -307,6 +329,7 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         {'vnuma_enabled': mock.sentinel.vnuma_enabled},
         {'configuration_root_dir': mock.sentinel.configuration_root_dir},
         {'host_shutdown_action': mock.sentinel.shutdown_action},
+        {'chassis_asset_tag': mock.sentinel.chassis_asset_tag2},
         {})
     @ddt.unpack
     @mock.patch.object(vmutils.VMUtils, '_modify_virtual_system')
@@ -319,7 +342,8 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
                        mock_set_mem, mock_set_vcpus,
                        mock_modify_virtual_system,
                        host_shutdown_action=None,
-                       configuration_root_dir=None, vnuma_enabled=None):
+                       configuration_root_dir=None, vnuma_enabled=None,
+                       chassis_asset_tag=None):
         mock_vmsettings = mock_lookup_vm_check.return_value
         self._vmutils.update_vm(
             mock.sentinel.vm_name, mock.sentinel.memory_mb,
@@ -328,7 +352,8 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
             mock.sentinel.dynamic_mem_ratio, configuration_root_dir,
             host_shutdown_action=host_shutdown_action,
             vnuma_enabled=vnuma_enabled,
-            snapshot_type=mock.sentinel.snap_type)
+            snapshot_type=mock.sentinel.snap_type,
+            chassis_asset_tag=chassis_asset_tag)
 
         mock_lookup_vm_check.assert_called_once_with(mock.sentinel.vm_name,
                                                      for_update=True)
@@ -356,6 +381,10 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         if vnuma_enabled:
             self.assertEqual(vnuma_enabled, mock_vmsettings.VirtualNumaEnabled)
 
+        if chassis_asset_tag:
+            self.assertEqual(chassis_asset_tag,
+                             mock_vmsettings.ChassisAssetTag)
+
         mock_set_vm_snap_type.assert_called_once_with(
             mock_vmsettings, mock.sentinel.snap_type)
 
@@ -715,6 +744,30 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self._vmutils._jobutils.remove_virt_resource.assert_called_once_with(
             mock_nic_data)
 
+    @mock.patch.object(vmutils.VMUtils, '_lookup_vm_check')
+    @mock.patch.object(_wqlutils, 'get_element_associated_class')
+    def test_get_vm_nics(self, mock_get_assoc, mock_lookup_vm):
+        vnics = self._vmutils._get_vm_nics(mock.sentinel.vm_name)
+
+        self.assertEqual(mock_get_assoc.return_value, vnics)
+        mock_lookup_vm.assert_called_once_with(mock.sentinel.vm_name)
+        mock_get_assoc.assert_called_once_with(
+            self._vmutils._compat_conn,
+            self._vmutils._SYNTHETIC_ETHERNET_PORT_SETTING_DATA_CLASS,
+            element_instance_id=mock_lookup_vm.return_value.InstanceId)
+
+    @mock.patch.object(vmutils.VMUtils, '_get_vm_nics')
+    def test_get_vm_nic_names(self, mock_get_vm_nics):
+        exp_nic_names = ['port1', 'port2']
+
+        mock_get_vm_nics.return_value = [
+            mock.Mock(ElementName=nic_name)
+            for nic_name in exp_nic_names]
+        nic_names = self._vmutils.get_vm_nic_names(mock.sentinel.vm_name)
+
+        self.assertEqual(exp_nic_names, nic_names)
+        mock_get_vm_nics.assert_called_once_with(mock.sentinel.vm_name)
+
     def test_set_vm_state(self):
         mock_vm = self._lookup_vm()
         mock_vm.RequestStateChange.return_value = (
@@ -1203,7 +1256,8 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self._vmutils.stop_vm_jobs(mock.sentinel.vm_name)
 
-        self._vmutils._jobutils.stop_jobs.assert_called_once_with(mock_vm)
+        self._vmutils._jobutils.stop_jobs.assert_called_once_with(
+            mock_vm, None)
 
     def test_set_secure_boot(self):
         vs_data = mock.MagicMock()
