@@ -14,9 +14,9 @@
 
 import ctypes
 import os
+from unittest import mock
 
 import ddt
-import mock
 import six
 
 from os_win import constants
@@ -32,6 +32,7 @@ class VHDUtilsTestCase(test_base.BaseTestCase):
     """Unit tests for the Hyper-V VHDUtils class."""
 
     _autospec_classes = [
+        vhdutils.diskutils.DiskUtils,
         vhdutils.win32utils.Win32Utils,
     ]
 
@@ -47,6 +48,8 @@ class VHDUtilsTestCase(test_base.BaseTestCase):
         self._mock_run = self._vhdutils._win32_utils.run_and_check_output
         self._run_args = self._vhdutils._virtdisk_run_args
 
+        self._disk_utils = self._vhdutils._disk_utils
+
         self.addCleanup(mock.patch.stopall)
 
     def _setup_lib_mocks(self):
@@ -817,23 +820,51 @@ class VHDUtilsTestCase(test_base.BaseTestCase):
             bool(attach_flag &
                  w_const.ATTACH_VIRTUAL_DISK_FLAG_PERMANENT_LIFETIME))
 
+    @ddt.data(True, False)
+    @mock.patch('os.path.exists')
     @mock.patch.object(vhdutils.VHDUtils, '_open')
-    def test_detach_virtual_disk(self, mock_open):
+    def test_detach_virtual_disk(self, exists, mock_open, mock_exists):
+        mock_exists.return_value = exists
         self._mock_run.return_value = w_const.ERROR_NOT_READY
 
         self._vhdutils.detach_virtual_disk(mock.sentinel.vhd_path)
 
-        mock_open.assert_called_once_with(
-            mock.sentinel.vhd_path,
-            open_access_mask=w_const.VIRTUAL_DISK_ACCESS_DETACH)
+        mock_exists.assert_called_once_with(mock.sentinel.vhd_path)
+        if exists:
+            mock_open.assert_called_once_with(
+                mock.sentinel.vhd_path,
+                open_access_mask=w_const.VIRTUAL_DISK_ACCESS_DETACH)
 
-        self._mock_run.assert_called_once_with(
-            vhdutils.virtdisk.DetachVirtualDisk,
-            mock_open.return_value,
-            0, 0,
-            ignored_error_codes=[w_const.ERROR_NOT_READY],
-            **self._run_args)
-        self._mock_close.assert_called_once_with(mock_open.return_value)
+            self._mock_run.assert_called_once_with(
+                vhdutils.virtdisk.DetachVirtualDisk,
+                mock_open.return_value,
+                0, 0,
+                ignored_error_codes=[w_const.ERROR_NOT_READY],
+                **self._run_args)
+            self._mock_close.assert_called_once_with(mock_open.return_value)
+        else:
+            mock_open.assert_not_called()
+
+    @ddt.data(True, False)
+    @mock.patch('os.path.exists')
+    @mock.patch.object(vhdutils.VHDUtils, '_open')
+    @mock.patch.object(vhdutils.VHDUtils, 'is_virtual_disk_file_attached')
+    def test_detach_virtual_disk_exc(self, is_attached, mock_is_attached,
+                                     mock_open, mock_exists):
+        # We'll try another approach before erroring out if the image cannot
+        # be opened (e.g. attached on a different host).
+        mock_exists.return_value = True
+        mock_is_attached.return_value = is_attached
+        mock_open.side_effect = exceptions.Win32Exception(message='fake exc')
+
+        if is_attached:
+            self.assertRaises(exceptions.Win32Exception,
+                              self._vhdutils.detach_virtual_disk,
+                              mock.sentinel.vhd_path)
+        else:
+            self._vhdutils.detach_virtual_disk(mock.sentinel.vhd_path)
+
+        mock_is_attached.assert_called_once_with(mock.sentinel.vhd_path)
 
     @mock.patch.object(vhdutils.VHDUtils, '_open')
     def test_get_virtual_disk_physical_path(self, mock_open):
@@ -867,3 +898,40 @@ class VHDUtilsTestCase(test_base.BaseTestCase):
             mock.ANY,
             mock.ANY,
             **self._run_args)
+
+    @ddt.data({},
+              {'exists': False},
+              {'open_fails': True})
+    @ddt.unpack
+    @mock.patch('os.path.exists')
+    @mock.patch.object(vhdutils.VHDUtils, 'get_vhd_info')
+    def test_is_virtual_disk_file_attached(self, mock_get_vhd_info,
+                                           mock_exists,
+                                           exists=True, open_fails=False):
+        mock_exists.return_value = exists
+        if open_fails:
+            mock_get_vhd_info.side_effect = exceptions.Win32Exception(
+                message="fake exc")
+        else:
+            mock_get_vhd_info.return_value = {
+                'IsLoaded': mock.sentinel.attached}
+
+        fallback = self._disk_utils.is_virtual_disk_file_attached
+        fallback.return_value = True
+
+        ret_val = self._vhdutils.is_virtual_disk_file_attached(
+            mock.sentinel.vhd_path)
+        exp_ret_val = True if exists else False
+
+        self.assertEqual(exp_ret_val, ret_val)
+        if exists:
+            mock_get_vhd_info.assert_called_once_with(
+                mock.sentinel.vhd_path,
+                [w_const.GET_VIRTUAL_DISK_INFO_IS_LOADED])
+        else:
+            mock_get_vhd_info.assert_not_called()
+
+        if exists and open_fails:
+            fallback.assert_called_once_with(mock.sentinel.vhd_path)
+        else:
+            fallback.assert_not_called()
