@@ -237,12 +237,29 @@ class VMUtils(baseutils.BaseUtilsVirt):
         vm = self._lookup_vm_check(vm_name, as_vssd=False)
         return vm.Name
 
-    def _set_vm_memory(self, vmsetting, memory_mb, memory_per_numa_node,
-                       dynamic_memory_ratio):
+    def get_vm_memory_info(self, vm_name):
+        vmsetting = self._lookup_vm_check(vm_name)
+        memory = self._get_vm_memory(vmsetting)
+
+        memory_info_dict = {
+            'DynamicMemoryEnabled': memory.DynamicMemoryEnabled,
+            'Reservation': memory.Reservation,
+            'Limit': memory.Limit,
+            'Weight': memory.Weight,
+            'MaxMemoryBlocksPerNumaNode': memory.MaxMemoryBlocksPerNumaNode,
+        }
+        return memory_info_dict
+
+    def _get_vm_memory(self, vmsetting):
         mem_settings = _wqlutils.get_element_associated_class(
             self._compat_conn, self._MEMORY_SETTING_DATA_CLASS,
             element_instance_id=vmsetting.InstanceID)[0]
 
+        return mem_settings
+
+    def _set_vm_memory(self, vmsetting, memory_mb, memory_per_numa_node,
+                       dynamic_memory_ratio):
+        mem_settings = self._get_vm_memory(vmsetting)
         max_mem = int(memory_mb)
         mem_settings.Limit = max_mem
 
@@ -297,7 +314,8 @@ class VMUtils(baseutils.BaseUtilsVirt):
                   configuration_root_dir=None, snapshot_dir=None,
                   host_shutdown_action=None, vnuma_enabled=None,
                   snapshot_type=None,
-                  is_planned_vm=False):
+                  is_planned_vm=False,
+                  chassis_asset_tag=None):
         vmsetting = self._lookup_vm_check(vm_name, for_update=True)
 
         if host_shutdown_action:
@@ -321,6 +339,9 @@ class VMUtils(baseutils.BaseUtilsVirt):
         if snapshot_type:
             self._set_vm_snapshot_type(vmsetting, snapshot_type)
 
+        if chassis_asset_tag:
+            vmsetting.ChassisAssetTag = chassis_asset_tag
+
         self._modify_virtual_system(vmsetting)
 
     def check_admin_permissions(self):
@@ -628,6 +649,18 @@ class VMUtils(baseutils.BaseUtilsVirt):
                       "to remove vm nic: '%s'. It may have been already "
                       "deleted.", nic_name)
 
+    def _get_vm_nics(self, vm_name):
+        vmsettings = self._lookup_vm_check(vm_name)
+        nics = _wqlutils.get_element_associated_class(
+            self._compat_conn,
+            self._SYNTHETIC_ETHERNET_PORT_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceId)
+        return nics
+
+    def get_vm_nic_names(self, vm_name):
+        nics = self._get_vm_nics(vm_name)
+        return [nic.ElementName for nic in nics]
+
     def soft_shutdown_vm(self, vm_name):
         vm = self._lookup_vm_check(vm_name, as_vssd=False)
         shutdown_component = self._conn.Msvm_ShutdownComponent(
@@ -985,9 +1018,9 @@ class VMUtils(baseutils.BaseUtilsVirt):
             pass
         return constants.VM_GEN_1
 
-    def stop_vm_jobs(self, vm_name):
+    def stop_vm_jobs(self, vm_name, timeout=None):
         vm = self._lookup_vm_check(vm_name, as_vssd=False)
-        self._jobutils.stop_jobs(vm)
+        self._jobutils.stop_jobs(vm, timeout)
 
     def enable_secure_boot(self, vm_name, msft_ca_required):
         """Enables Secure Boot for the instance with the given name.
@@ -1035,7 +1068,8 @@ class VMUtils(baseutils.BaseUtilsVirt):
         # need to accept Msvm_DiskDrive object paths for image files as well,
         # an extra check will be needed, but that may lead to some other
         # inconsistencies.
-        is_physical = r'root\virtualization\v2:Msvm_DiskDrive' in drive_path
+        is_physical = (r'root\virtualization\v2:Msvm_DiskDrive'.lower() in
+                       drive_path.lower())
         drive = self._get_mounted_disk_resource_from_path(
             drive_path, is_physical=is_physical)
 
