@@ -13,19 +13,28 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import re
+
+from oslo_log import log as logging
+import six
+
 from os_win._i18n import _
+from os_win import _utils
 from os_win import constants
 from os_win import exceptions
 from os_win.utils import _wqlutils
 from os_win.utils.compute import vmutils
 from oslo_utils import units
 
+LOG = logging.getLogger(__name__)
 
-class VMUtils10(vmutils.VMUtils):
+
+class VMUtils10(vmutils.VMUtils6_3):
 
     _UEFI_CERTIFICATE_AUTH = 'MicrosoftUEFICertificateAuthority'
     _SERIAL_PORT_SETTING_DATA_CLASS = "Msvm_SerialPortSettingData"
     _SECURITY_SETTING_DATA = 'Msvm_SecuritySettingData'
+    _PCI_EXPRESS_SETTING_DATA = 'Msvm_PciExpressSettingData'
     _MSPS_NAMESPACE = '//%s/root/msps'
 
     _remote_fx_res_map = {
@@ -76,6 +85,21 @@ class VMUtils10(vmutils.VMUtils):
             self._sec_svc_attr = self._conn.Msvm_SecurityService()[0]
         return self._sec_svc_attr
 
+    def set_nested_virtualization(self, vm_name, state):
+        """Enables nested virtualization for the given VM.
+
+        :param vm_name: the name of the VM.
+        :param state: boolean, if True, nested virtualization will be enabled,
+            disabled otherwise.
+        """
+        vmsettings = self._lookup_vm_check(vm_name)
+        procsettings = _wqlutils.get_element_associated_class(
+            self._conn, self._PROCESSOR_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceID)[0]
+
+        procsettings.ExposeVirtualizationExtensions = state
+        self._jobutils.modify_virt_resource(procsettings)
+
     def vm_gen_supports_remotefx(self, vm_gen):
         """RemoteFX is supported on both generation 1 and 2 virtual
 
@@ -90,26 +114,16 @@ class VMUtils10(vmutils.VMUtils):
                                   vram_bytes=None):
         super(VMUtils10, self)._validate_remotefx_params(monitor_count,
                                                          max_resolution)
-        if vram_bytes not in self._remotefx_vram_vals:
+        if vram_bytes and vram_bytes not in self._remotefx_vram_vals:
             raise exceptions.HyperVRemoteFXException(
                 _("Unsuported RemoteFX VRAM value: %(requested_value)s."
                   "The supported VRAM values are: %(supported_values)s") %
                 {'requested_value': vram_bytes,
                  'supported_values': self._remotefx_vram_vals})
 
-    def _add_3d_display_controller(self, vm, monitor_count,
-                                   max_resolution, vram_bytes=None):
-        synth_3d_disp_ctrl_res = self._get_new_resource_setting_data(
-            self._SYNTH_3D_DISP_CTRL_RES_SUB_TYPE,
-            self._SYNTH_3D_DISP_ALLOCATION_SETTING_DATA_CLASS)
-
-        synth_3d_disp_ctrl_res.MaximumMonitors = monitor_count
-        synth_3d_disp_ctrl_res.MaximumScreenResolution = max_resolution
-
+    def _set_remotefx_vram(self, remotefx_disp_ctrl_res, vram_bytes):
         if vram_bytes:
-            synth_3d_disp_ctrl_res.VRAMSizeBytes = unicode(vram_bytes)
-
-        self._jobutils.add_virt_resource(synth_3d_disp_ctrl_res, vm)
+            remotefx_disp_ctrl_res.VRAMSizeBytes = six.text_type(vram_bytes)
 
     def _vm_has_s3_controller(self, vm_name):
         return self.get_vm_generation(vm_name) == constants.VM_GEN_1
@@ -190,3 +204,95 @@ class VMUtils10(vmutils.VMUtils):
         if security_profile:
             return security_profile[0].EncryptStateAndVmMigrationTraffic
         return False
+
+    def add_pci_device(self, vm_name, vendor_id, product_id):
+        """Adds the given PCI device to the given VM.
+
+        :param vm_name: the name of the VM to which the PCI device will be
+            attached to.
+        :param vendor_id: the PCI device's vendor ID.
+        :param product_id: the PCI device's product ID.
+        :raises exceptions.PciDeviceNotFound: if there is no PCI device
+            identifiable by the given vendor_id and product_id, or it was
+            already assigned.
+        """
+        vmsettings = self._lookup_vm_check(vm_name)
+        pci_setting_data = self._get_new_setting_data(
+            self._PCI_EXPRESS_SETTING_DATA)
+        pci_device = self._get_assignable_pci_device(vendor_id, product_id)
+        pci_setting_data.HostResource = [pci_device.path_()]
+
+        self._jobutils.add_virt_resource(pci_setting_data, vmsettings)
+
+    def _get_assignable_pci_device(self, vendor_id, product_id):
+        pci_devices = self._conn.Msvm_PciExpress()
+
+        pattern = re.compile(
+            "^(.*)VEN_%(vendor_id)s&DEV_%(product_id)s&(.*)$" % {
+                'vendor_id': vendor_id, 'product_id': product_id},
+            re.IGNORECASE)
+        for dev in pci_devices:
+            if pattern.match(dev.DeviceID):
+                # NOTE(claudiub): if the given PCI device is already assigned,
+                # the pci_devices list will contain PCI device with the same
+                # LocationPath.
+                pci_devices_found = [d for d in pci_devices if
+                                     d.LocationPath == dev.LocationPath]
+
+                LOG.debug('PCI devices found: %s',
+                          [d.DeviceID for d in pci_devices_found])
+
+                # device is not in use by other VM
+                if len(pci_devices_found) == 1:
+                    return pci_devices_found[0]
+
+        raise exceptions.PciDeviceNotFound(vendor_id=vendor_id,
+                                           product_id=product_id)
+
+    def remove_pci_device(self, vm_name, vendor_id, product_id):
+        """Removes the given PCI device from the given VM.
+
+        :param vm_name: the name of the VM from which the PCI device will be
+            attached from.
+        :param vendor_id: the PCI device's vendor ID.
+        :param product_id: the PCI device's product ID.
+        """
+        vmsettings = self._lookup_vm_check(vm_name)
+
+        pattern = re.compile(
+            "^(.*)VEN_%(vendor_id)s&DEV_%(product_id)s&(.*)$" % {
+                'vendor_id': vendor_id, 'product_id': product_id},
+            re.IGNORECASE)
+
+        pci_sds = _wqlutils.get_element_associated_class(
+            self._conn, self._PCI_EXPRESS_SETTING_DATA,
+            vmsettings.InstanceID)
+        pci_sds = [sd for sd in pci_sds if pattern.match(sd.HostResource[0])]
+
+        if pci_sds:
+            self._jobutils.remove_virt_resource(pci_sds[0])
+        else:
+            LOG.debug("PCI device with vendor ID %(vendor_id)s and "
+                      "%(product_id)s is not attached to %(vm_name)s",
+                      {'vendor_id': vendor_id, 'product_id': product_id,
+                       'vm_name': vm_name})
+
+    def remove_all_pci_devices(self, vm_name):
+        """Removes all the PCI devices from the given VM.
+
+        :param vm_name: the name of the VM from which all the PCI devices will
+            be detached from.
+        """
+        vmsettings = self._lookup_vm_check(vm_name)
+
+        pci_sds = _wqlutils.get_element_associated_class(
+            self._conn, self._PCI_EXPRESS_SETTING_DATA,
+            vmsettings.InstanceID)
+
+        if pci_sds:
+            self._jobutils.remove_multiple_virt_resources(pci_sds)
+
+    @_utils.required_vm_version(min_version=constants.VM_VERSION_6_2)
+    def _set_vm_snapshot_type(self, vmsettings, snapshot_type):
+        # We expect the caller to actually push the vmsettings update.
+        vmsettings.UserSnapshotType = snapshot_type
