@@ -15,7 +15,6 @@
 
 import ctypes
 import struct
-import sys
 
 from eventlet import patcher
 from oslo_log import log as logging
@@ -34,11 +33,7 @@ kernel32 = w_lib.get_shared_lib_handle(w_lib.KERNEL32)
 
 LOG = logging.getLogger(__name__)
 
-# Avoid using six.moves.queue as we need a non monkey patched class
-if sys.version_info > (3, 0):
-    Queue = patcher.original('queue')
-else:
-    Queue = patcher.original('Queue')
+Queue = patcher.original('queue')
 
 WAIT_PIPE_DEFAULT_TIMEOUT = 5  # seconds
 WAIT_IO_COMPLETION_TIMEOUT = 2 * units.k
@@ -61,6 +56,31 @@ class IOUtils(object):
                       eventlet_nonblocking_mode=eventlet_blocking_mode)
         return self._win32_utils.run_and_check_output(*args, **kwargs)
 
+    def create_pipe(self, security_attributes=None, size=0,
+                    inherit_handle=False):
+        """Create an anonymous pipe.
+
+        The main advantage of this method over os.pipe is that it allows
+        creating inheritable pipe handles (which is flawed on most Python
+        versions).
+        """
+        r = wintypes.HANDLE()
+        w = wintypes.HANDLE()
+
+        if inherit_handle and not security_attributes:
+            security_attributes = wintypes.SECURITY_ATTRIBUTES()
+            security_attributes.bInheritHandle = inherit_handle
+            security_attributes.nLength = ctypes.sizeof(security_attributes)
+
+        self._run_and_check_output(
+            kernel32.CreatePipe,
+            ctypes.byref(r),
+            ctypes.byref(w),
+            ctypes.byref(security_attributes) if security_attributes else None,
+            size)
+
+        return r.value, w.value
+
     @_utils.retry_decorator(exceptions=exceptions.Win32IOException,
                             max_sleep_time=2)
     def wait_named_pipe(self, pipe_name, timeout=WAIT_PIPE_DEFAULT_TIMEOUT):
@@ -69,8 +89,8 @@ class IOUtils(object):
                                    ctypes.c_wchar_p(pipe_name),
                                    timeout * units.k)
 
-    def open(self, path, desired_access=None, share_mode=None,
-             creation_disposition=None, flags_and_attributes=None):
+    def open(self, path, desired_access=0, share_mode=0,
+             creation_disposition=0, flags_and_attributes=0):
         error_ret_vals = [w_const.INVALID_HANDLE_VALUE]
         handle = self._run_and_check_output(kernel32.CreateFileW,
                                             ctypes.c_wchar_p(path),
@@ -155,6 +175,18 @@ class IOUtils(object):
                                    completion_routine)
         self._wait_io_completion(overlapped_structure.hEvent)
 
+    def read_file(self, handle, buff, num_bytes, overlapped_structure=None):
+        # Similar to IOUtils.read, but intended for synchronous operations.
+        num_bytes_read = wintypes.DWORD(0)
+        overlapped_structure_ref = (
+            ctypes.byref(overlapped_structure) if overlapped_structure
+            else None)
+        self._run_and_check_output(kernel32.ReadFile,
+                                   handle, buff, num_bytes,
+                                   ctypes.byref(num_bytes_read),
+                                   overlapped_structure_ref)
+        return num_bytes_read.value
+
     def write(self, handle, buff, num_bytes,
               overlapped_structure, completion_routine):
         self._reset_event(overlapped_structure.hEvent)
@@ -164,6 +196,18 @@ class IOUtils(object):
                                    completion_routine)
         self._wait_io_completion(overlapped_structure.hEvent)
 
+    def write_file(self, handle, buff, num_bytes, overlapped_structure=None):
+        # Similar to IOUtils.write, but intended for synchronous operations.
+        num_bytes_written = wintypes.DWORD(0)
+        overlapped_structure_ref = (
+            ctypes.byref(overlapped_structure) if overlapped_structure
+            else None)
+        self._run_and_check_output(kernel32.WriteFile,
+                                   handle, buff, num_bytes,
+                                   ctypes.byref(num_bytes_written),
+                                   overlapped_structure_ref)
+        return num_bytes_written.value
+
     @classmethod
     def get_buffer(cls, buff_size, data=None):
         buff = (ctypes.c_ubyte * buff_size)()
